package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;

import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.MarkerSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.RejectHistoryService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.utils.RequestUtils;

@Controller
@RequestMapping("/admin/exam/reject")
public class RejectHistoryController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(RejectHistoryController.class);

    @Autowired
    private RejectHistoryService historyService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private UserService userService;

    @Autowired
    private MarkLibraryService libraryService;

    @Logging(menu = "打回任务查询", type = LogType.QUERY)
    @RequestMapping(value = "/list")
    public String list(Model model, HttpServletRequest request, MarkLibrarySearchQuery query) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        query.setExamId(examId);
        query.addStatus(LibraryStatus.REJECTED);
        query = libraryService.findByQuery(query);
        for (MarkLibrary library : query.getResult()) {
            if (library.getMarkerId() != null) {
                Marker marker = markerService.findById(library.getMarkerId());
                User user = userService.findById(marker.getUserId());
                library.setMarkerLoginName(user.getLoginName() + "/" + user.getName());
            }
            if (library.getHeaderId() != null) {
                User user = userService.findById(library.getHeaderId());
                library.setHeaderLoginName(user.getLoginName() + "/" + user.getName());
            }
            MarkGroup group = groupService.findOne(examId, library.getSubjectCode(), library.getGroupNumber());
            group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                    group.getSubjectCode(), false, group.getNumber()));
            library.setTitle(group.getTitle());
        }

        model.addAttribute("query", query);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        List<Marker> markerList = new ArrayList<Marker>();
        if (query.getSubjectCode() != null) {
            MarkerSearchQuery mQuery = new MarkerSearchQuery();
            mQuery.setExamId(examId);
            mQuery.setSubjectCode(query.getSubjectCode());
            mQuery.setPageSize(Integer.MAX_VALUE);
            markerList = markerService.findByQuery(mQuery).getResult();
            for (Marker marker : markerList) {
                marker.setLoginName(userService.findById(marker.getUserId()).getLoginName());
            }
        }
        model.addAttribute("markerList", markerList);
        return "modules/exam/rejectList";
    }

    @Logging(menu = "打回任务查询", type = LogType.QUERY)
    @RequestMapping(value = "/history")
    public String history(Model model, HttpServletRequest request, MarkLibrarySearchQuery query) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        query.setExamId(examId);
        query.addStatus(LibraryStatus.REJECTED);
        model.addAttribute("query", query);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        List<Marker> markerList = new ArrayList<Marker>();
        if (query.getSubjectCode() != null) {
            MarkerSearchQuery mQuery = new MarkerSearchQuery();
            mQuery.setExamId(examId);
            mQuery.setSubjectCode(query.getSubjectCode());
            mQuery.setPageSize(Integer.MAX_VALUE);
            markerList = markerService.findByQuery(mQuery).getResult();
            for (Marker marker : markerList) {
                marker.setLoginName(userService.findById(marker.getUserId()).getLoginName());
            }
        }
        model.addAttribute("markerList", markerList);
        return "modules/exam/rejectList";
    }
}
